@@ -4,8 +4,8 @@
 #'
 #' @docType class
 #' @section Methods:
-#' - `new(tenant, app, ...)`: Initialize a new Microsoft Graph connection with the given credentials. See 'Authentication` for more details.
-#' - `create_app(name, ..., password=NULL, password_duration=1, create_service_principal=TRUE)`: Creates a new registered app in Azure Active Directory. By default the app will have a randomly generated strong password with a duration of 1 year, and an associated service principal will also be created. To skip assigning a password, set `password=FALSE`.
+#' - `new(tenant, app, ...)`: Initialize a new Microsoft Graph connection with the given credentials. See 'Authentication' for more details.
+#' - `create_app(name, ..., password=NULL, password_duration=1, certificate=NULL, create_service_principal=TRUE)`: Creates a new registered app in Azure Active Directory. See 'App creation' below.
 #' - `get_app(app_id, object_id)`: Retrieves an existing registered app, via either its app ID or object ID.
 #' - `delete_app(app_id, object_id, confirm=TRUE)`: Deletes an existing registered app. Any associated service principal will also be deleted.
 #' - `create_service_principal(app_id, ...)`: Creates a service principal for a registered app.
@@ -33,6 +33,13 @@
 #' - `config_file`: Optionally, a JSON file containing any of the arguments listed above. Arguments supplied in this file take priority over those supplied on the command line. You can also use the output from the Azure CLI `az ad sp create-for-rbac` command.
 #' - `token`: Optionally, an OAuth 2.0 token, of class [AzureAuth::AzureToken]. This allows you to reuse the authentication details for an existing session. If supplied, all other arguments will be ignored.
 #'
+#' @section App creation:
+#' The `create_app` method creates a new registered app. By default, a new app will have a randomly generated strong password with duration of 1 year. To skip assigning a password, set the `password` argument to FALSE.
+#'
+#' The `certificate` argument allows authenticating via a certificate instead of a password. This should be a character string containing the certificate public key (aka the CER file). Alternatively it can be an list, or an object of class `AzureKeyVault::stored_cert` representing a certificate stored in an Azure Key Vault. See the examples below.
+#'
+#' A new app will also have a service principal created for it by default. To disable this, set `create_service_principal=FALSE`.
+#'
 #' @seealso
 #' [create_graph_login], [get_graph_login]
 #'
@@ -62,6 +69,10 @@
 #' # ... but better to call the object's delete method directly
 #' app$delete()
 #'
+#' # create an app with authentication via a certificate
+#' cert <- readLines("mycert.cer")
+#' gr$create_app("mycertapp", password=FALSE, certificate=cert)
+#'
 #' }
 #' @format An R6 object of class `ms_graph`.
 #' @export
@@ -73,7 +84,7 @@ public=list(
     token=NULL,
 
     # authenticate and get subscriptions
-    initialize=function(tenant, app=.az_cli_app_id, password=NULL, username=NULL, auth_type=NULL,
+    initialize=function(tenant="common", app=.az_cli_app_id, password=NULL, username=NULL, auth_type=NULL,
                         host="https://graph.microsoft.com/", aad_host="https://login.microsoftonline.com/",
                         config_file=NULL, token=NULL)
     {
@@ -81,7 +92,12 @@ public=list(
         {
             self$host <- if(token$version == 1)
                 token$resource
-            else token$scope
+            else
+            {
+                scope <- httr::parse_url(token$scope[1])
+                scope$path <- NULL
+                httr::build_url(scope)
+            }
             self$tenant <- token$tenant
             self$token <- token
             return(NULL)
@@ -94,6 +110,7 @@ public=list(
             if(!is.null(conf$app)) app <- conf$app
             if(!is.null(conf$auth_type)) auth_type <- conf$auth_type
             if(!is.null(conf$password)) password <- conf$password
+            if(!is.null(conf$username)) username <- conf$username
             if(!is.null(conf$graph_host)) host <- conf$graph_host
             if(!is.null(conf$aad_host)) aad_host <- conf$aad_host
         }
@@ -114,7 +131,8 @@ public=list(
         NULL
     },
 
-    create_app=function(name, ..., password=NULL, password_duration=1, create_service_principal=TRUE)
+    create_app=function(name, ..., password=NULL, password_duration=1, certificate=NULL,
+        create_service_principal=TRUE)
     {
         properties <- list(displayName=name, ...)
         if(is.null(password) || password != FALSE)
@@ -140,6 +158,22 @@ public=list(
             ))
         }
 
+        if(!is_empty(certificate))
+        {
+            key <- if(is.character(certificate))
+                certificate
+            else if(is.list(certificate) || inherits(certificate, "stored_cert"))
+                certificate$cer
+
+            properties <- modifyList(properties, list(
+                keyCredentials=list(list(
+                    key=key,
+                    type="AsymmetricX509Cert",
+                    usage="verify"
+                ))
+            ))
+        }
+
         res <- az_app$new(
             self$token,
             self$tenant,
